#!/usr/bin/env python
# -*- coding: UTF8 -*-
#
# XAMPP Control Panel
# Written by Jono Bacon (jono@jonobacon.org)
#
# Licensed under the GNU Public License v2
# 
# This program is free software. See the included LICENSE file for details.
#
# Python module xamppcontrolpanel.py
# Autogenerated from xamppcontrolpanel.glade
# Generated on Wed Aug 24 00:37:20 2005

# Warning: Do not delete or modify comments related to context
# They are required to keep user's code

import os
import os.path
import string
import gtk
import commands
import posix
import ConfigParser
import threading
import time
import gobject
from SimpleGladeApp import SimpleGladeApp

glade_dir = ""

# Put your modules and data here

# From here through main() codegen inserts/updates a class for
# every top-level widget in the .glade file.

class MainWindow(SimpleGladeApp):
    def __init__(self, path="xampp-control-panel.glade", root="MainWindow", domain=None, **kwargs):
        path = os.path.join(glade_dir, path)
        SimpleGladeApp.__init__(self, path, root, domain, **kwargs)
		
        # default location of config file for this user (~/.xampp_cp);
        # config file has format specified for ConfigParser module, e.g.
        #
        # [main]
        # xampp_path=/opt/lampp
        self.CONFIG_FILE_PATH = posix.environ['HOME'] + "/.xampp_cp"
	
        # set up the stop buttons that are hidden at the start - the execute buttons were created in glade :)
        self.mainStopButton = gtk.Button(None, gtk.STOCK_MEDIA_STOP)
        self.mainStopButton.set_border_width(5)
        self.mainStopButton.connect("clicked", self.on_mainStopButton_clicked)
        self.vbox1.pack_start(self.mainStopButton, True, False)
        self.vbox1.reorder_child(self.mainStopButton, 0)
        self.mainStopButton.hide()
        
        self.apacheStopButton = gtk.Button(None, gtk.STOCK_MEDIA_STOP)
        self.apacheStopButton.set_border_width(0)
        self.apacheStopButton.connect("clicked", self.on_apacheStopButton_clicked)
        self.table1.attach(self.apacheStopButton, 2, 3, 0, 1)
        self.apacheStopButton.hide()

        self.mysqlStopButton = gtk.Button(None, gtk.STOCK_MEDIA_STOP)
        self.mysqlStopButton.set_border_width(0)
        self.mysqlStopButton.connect("clicked", self.on_mysqlStopButton_clicked)
        self.table1.attach(self.mysqlStopButton, 2, 3, 1, 2)
        self.mysqlStopButton.hide()
	    
        self.proftpdStopButton = gtk.Button(None, gtk.STOCK_MEDIA_STOP)
        self.proftpdStopButton.set_border_width(0)
        self.proftpdStopButton.connect("clicked", self.on_proftpdStopButton_clicked)
        self.table1.attach(self.proftpdStopButton, 2, 3, 2, 3)
        self.proftpdStopButton.hide()

        # get path configuration from .xcp file
        xampppath = self.get_xampp_path()
        if xampppath == "":
            self.set_xampp_path("/opt/lampp")
        
        # initialise the interface
        # TODO: need to poll the interface every UPDATE_INTERVAL seconds
        self.set_interface()

        self.buttonval = 0

    def new(self):
        print "A new MainWindow has been created"
	
    # get the path info from the config file, or return "" if not set yet
    def get_xampp_path(self):
        xampp_path = ""
        config_parser = self.get_config_parser()
        if config_parser.has_option('main', 'xampp_path'):
            xampp_path = config_parser.get('main', 'xampp_path')
        return xampp_path
	 
    # set the path info in the config file
    def set_xampp_path(self, path_to_xampp):
        config_parser = self.get_config_parser()
        if not config_parser.has_section('main'):
            config_parser.add_section('main')
            config_parser.set('main', 'xampp_path', path_to_xampp)
            config_parser.write(file(self.CONFIG_FILE_PATH, 'w'))
            
    # return a string representing a call to the xampp script
    def get_xampp_command(self, command):
        xampp_script = os.path.join(self.get_xampp_path(), "lampp")
        xampp_command = "%s %s" % (xampp_script, command)
        return xampp_command
	    
    # get a ConfigParser instance; will create the config file
    # if it doesn't already exist
    def get_config_parser(self):
        if not os.path.exists(self.CONFIG_FILE_PATH):
            config_file = open(self.CONFIG_FILE_PATH, 'w')
            config_file.close()
        config_parser = ConfigParser.ConfigParser()
        config_parser.readfp(open(self.CONFIG_FILE_PATH))
        return config_parser
        
    # get a dictionary of service statues by calling "lampp statusraw";
    # returns a dictionary like this:
    # {'APACHE':'RUNNING', 'MYSQL':'NOTRUNNING', 'PROFTPD':'DEACTIVATED'}
    def get_statuses(self):
        statuses = {}
        raw_status = commands.getoutput(self.get_xampp_command("statusraw"))
        lines = string.split(raw_status, "\n")
        
        # ignore first line, and create a dictionary of service/status
        # pairs by splitting each line around the central whitespace
        statuses = dict([string.split(line, " ") for line in lines[1:]])
        return statuses
    
    # set interface buttons and labels for all services
    def set_interface(self):
        # fetch statuses as a dictionary, keyed by service name
        all_statuses = self.get_statuses()
    
        # apache
        self.set_interface_for_service(all_statuses['APACHE'], self.apacheExButton, 
        self.apacheStopButton, self.apacheStatus)
		
        # mysql
        self.set_interface_for_service(all_statuses['MYSQL'], self.mysqlExButton,
        self.mysqlStopButton, self.mysqlStatus)

        # proftpd
        self.set_interface_for_service(all_statuses['PROFTPD'], self.proftpdExButton,
        self.proftpdStopButton, self.proftpdStatus)
        
        # main start and stop buttons
        if self.apacheExButton.get_property('visible') and self.mysqlExButton.get_property('visible') and self.proftpdExButton.get_property('visible'):
            self.mainStopButton.hide()
            self.mainStartButton.show()
        else:
            self.mainStopButton.show()
            self.mainStartButton.hide()

    # set the buttons and label for a single service
    def set_interface_for_service(self, service_status, execute_btn, stop_btn,
    status_label):
        if service_status == 'RUNNING':
            execute_btn.hide()
            stop_btn.show()
            status_label.set_text("RUNNING")
        else:
            execute_btn.show()
            stop_btn.hide()
            status_label.set_text("STOPPED")    

    # button handlers
    def on_mainStartButton_clicked(self, widget, *args):
        print "Main execute button clicked"
        os.popen(self.get_xampp_command("start"))

    def on_mainStopButton_clicked(self, widget, *args):
        print "Main stop button clicked"
        os.popen(self.get_xampp_command("stop"))

    def on_apacheButton_clicked(self, widget, *args):
        print "Apache execute button clicked"
        os.popen(self.get_xampp_command("startapache"))

    def on_apacheStopButton_clicked(self, widget, *args):
        print "Apache stop button clicked"
        os.popen(self.get_xampp_command("stopapache"))

    def on_mysqlButton_clicked(self, widget, *args):
        print "MySQL execute button clicked"
        os.popen(self.get_xampp_command("startmysql"))

    def on_mysqlStopButton_clicked(self, widget, *args):
        print "MySQL stop button clicked"
        os.popen(self.get_xampp_command("stopmysql"))

    def on_proftpdButton_clicked(self, widget, *args):
        print "ProFTPD execute button clicked"
        os.popen(self.get_xampp_command("startftp"))

    def on_proftpdStopButton_clicked(self, widget, *args):
        print "ProFTPD stop button clicked"
        os.popen(self.get_xampp_command("stopftp"))
        
    def on_prefsButton_clicked(self, widget, *args):
        print "Preferences clicked"
        self.prefsswindow = PrefsWindow("xampp-control-panel.glade", "PrefsWindow", parent=self)

    def on_cancelButton_clicked(self, widget, *args):
        print "Quitting"
        self.gtk_main_quit()

class PrefsWindow(SimpleGladeApp):
    def __init__(self, path="xamppcontrolpanel.glade", root="PrefsWindow", domain=None, **kwargs):
        path = os.path.join(glade_dir, path)
        SimpleGladeApp.__init__(self, path, root, domain, **kwargs)
        self.pathTextBox.set_text(self.parent.get_xampp_path())

    def new(self):
        print "A new PrefsWindow has been created"

    # handler for file chooser - select path to XAMPP installation
    def on_browseButton_clicked(self, widget, *args):
        fc = gtk.FileChooserDialog(
            title = 'Select the XAMPP installation directory',
            action = gtk.FILE_CHOOSER_ACTION_OPEN,
            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK)
        )
        fc.set_filename(self.pathTextBox.get_text())
        response = fc.run()
        if response == gtk.RESPONSE_OK:
            self.pathTextBox.set_text(fc.get_filename())
        fc.destroy()

    def on_applyButton_clicked(self, widget, *args):
        self.client.set_string("/apps/xampp-control-panel/xampp-path", self.pathTextBox.get_text())
        xampppath = self.get_xampp_path()
        if xampppath == "":
            self.parent.apacheExButton.set_sensitive(False)
            self.parent.mysqlExButton.set_sensitive(False)
            self.parent.proftpdExButton.set_sensitive(False)
        else:
            self.parent.apacheExButton.set_sensitive(True)
            self.parent.mysqlExButton.set_sensitive(True)
            self.parent.proftpdExButton.set_sensitive(True)
        self.PrefsWindow.destroy()

    def on_cancelButton_clicked(self, widget, *args):
        print "on_cancelButton_clicked called with self.%s" % widget.get_name()
        self.PrefsWindow.destroy()

# thread which runs in the background and updates the interface periodically
class StatusMonitor(threading.Thread):
    def __init__(self, main_window):
        super(StatusMonitor, self).__init__()
        self.main_window = main_window
        self.quit = False
        self.UPDATE_INTERVAL = 0.5

    def update_main_window(self):
        self.main_window.set_interface()

    def run(self):
        while not self.quit:
            gobject.idle_add(self.update_main_window)
            time.sleep(self.UPDATE_INTERVAL)

if __name__ == "__main__":
    gobject.threads_init()
    main_window = MainWindow()
    sm = StatusMonitor(main_window)
    sm.start()
    main_window.run()
    sm.quit = True
